/*************************************************** */
/* Rule Set Based Access Control                     */
/*                                                   */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/*                                                   */
/* Last modified: 29/Sep/2005                        */
/*************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <grp.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/resource.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include "nls.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

  char * progname;
  rsbac_list_repl_partner_number_t id;
  char * name = NULL;
  char * algo = NULL;
  char key[RSBAC_LIST_REPL_CRYPTKEY_LEN];
  __u32 crypt_key_len = 0;
  int set_key = 0;
  __u32 ip;
  int set_ip = 0;
  __u16 port;
  int set_port = 0;
  __u32 add_flags = 0;
  __u32 remove_flags = 0;
  rsbac_time_t ttl = 0;
  int set_ttl = 0;
  int verbose = 0;
  rsbac_list_ta_number_t ta_number = 0;

void use(void)
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
      printf(gettext("Use: %s [flags] id\n"), progname);
      printf(gettext(" id   = partner id to use,\n"));
      printf(gettext(" -n name = set new name,\n"));
      printf(gettext(" -i ip = set new partner ip,\n"));
      printf(gettext(" -p port = change partner port,\n"));
      printf(gettext(" -C algo = name of encryption algorithm (default is none),\n"));
      printf(gettext(" -K key = encryption key in hex notation,\n"));
      printf(gettext(" -a = activate,\n"));
      printf(gettext(" -A = deactivate,\n"));
      printf(gettext(" -m = partner is a master for this host,\n"));
      printf(gettext(" -M = partner is not a master for this host,\n"));
      printf(gettext(" -s = partner is a slave of this host,\n"));
      printf(gettext(" -S = partner is not a slave of this host,\n"));
      printf(gettext(" -t = set relative time-to-live in secs\n"));
      printf(gettext(" -T = set absolute time-to-live in secs\n"));
      printf(gettext(" -D = set relative time-to-live in days\n"));
      printf(gettext(" -N ta = transaction number (default = value of RSBAC_TA, if set, or 0)\n"));
    }

int key_read(char * to, char * from)
  {
    char * f = from;
    char * t = to;
    char   tmp[3];
    int i;
    int res = 0;

    if(strlen(from) > RSBAC_LIST_REPL_CRYPTKEY_LEN * 2)
      {
        fprintf(stderr, "Key is too long!\n");
        return -RSBAC_EINVALIDVALUE;
      }
    tmp[2] = 0;
    while(f[0] && f[1])
      {
        tmp[0] = f[0];
        tmp[1] = f[1];
        i = strtoul(tmp, 0, 16);
        if(i < 0 || i > 255)
          return -RSBAC_EINVALIDVALUE;
        *t = i;
        t++;
        f += 2;
        res++;
      }
    return res;
  }

int env_init(void)
{
	int res;

	struct rlimit rlimit;

	rlimit.rlim_cur = 0;
	rlimit.rlim_max = 0;
	res = setrlimit(RLIMIT_CORE, &rlimit);
	if (res)
		fprintf(stderr, gettext("Warning: unable to set resource limits.\n"));	
	
	return res;
}

int main(int argc, char ** argv)
{
  int res = 0;
  struct rsbac_list_repl_partner_entry_t entry;

  env_init();
  locale_init();

  progname = argv[0];
  memset(&entry, 0, sizeof(entry));
  {
    char * env = getenv("RSBAC_TA");

    if(env)
      ta_number = strtoul(env,0,0);
  }
  res = mlock(&entry, sizeof(entry));
  if (res)
	  fprintf(stderr, gettext("Warning: replication parter data was not locked into physical memory.\n"));
  
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'h':
                use();
                return 0;
              case 'v':
                verbose++;
                break;
              case 'a':
                add_flags |= REPL_flag_active;
                break;
              case 'm':
                add_flags |= REPL_flag_master;
                break;
              case 's':
                add_flags |= REPL_flag_slave;
                break;
              case 'A':
                remove_flags |= REPL_flag_active;
                break;
              case 'M':
                remove_flags |= REPL_flag_master;
                break;
              case 'S':
                remove_flags |= REPL_flag_slave;
                break;
              case 'K':
                if(argc > 2)
                  {
                    res = key_read(key, argv[2]);
                    error_exit(res);
                    crypt_key_len = res;
                    set_key = 1;
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'C':
                if(argc > 2)
                  {
                    algo = argv[2];
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'n':
                if(argc > 2)
                  {
                    name = argv[2];
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'i':
                if(argc > 2)
                  {
                    struct in_addr addr;

                    error_exit(inet_aton(argv[2], &addr));
                    ip = addr.s_addr;
                    set_ip = 1;
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 'p':
                if(argc > 2)
                  {
                    port = strtoul(argv[2], 0, 0);
                    set_port = 1;
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing argument for parameter %c\n"), progname, *pos);
                break;
              case 't':
                if(argc > 2)
                  {
                    ttl = strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'D':
                if(argc > 2)
                  {
                    ttl = 86400 * strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'T':
                if(argc > 2)
                  {
                    rsbac_time_t now = time(NULL);
                    ttl = strtoul(argv[2], 0, 10);
                    if(ttl > now)
                      {
                        ttl -= now;
                        argc--;
                        argv++;
                      }
                    else
                      {
                        fprintf(stderr,
                                gettext("%s: ttl value for parameter %c is in the past, exiting\n"), progname, *pos);
                        exit(1);
                      }
                  }
                else
                  fprintf(stderr, gettext("%s: missing ttl value for parameter %c\n"), progname, *pos);
                break;
              case 'N':
                if(argc > 2)
                  {
                    ta_number = strtoul(argv[2], 0, 10);
                    argc--;
                    argv++;
                  }
                else
                  {
                    fprintf(stderr, gettext("%s: missing transaction number value for parameter %c\n"), progname, *pos);
                    exit(1);
                  }
                break;

              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
                exit(1);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if (argc > 1)
    {
      int i;
      union rsbac_list_repl_mod_data_t data;

      id = strtoul(argv[1], 0, 0);
      if(!id)
        error_exit(-RSBAC_EINVALIDVALUE);
      if(verbose)
        printf("Modifying replication partner %u",
               id);
      if(name)
        {
          strncpy(data.string, name, RSBAC_LIST_REPL_NAME_LEN);
          data.string[RSBAC_LIST_REPL_NAME_LEN - 1] = 0;
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_name, &data);
          error_exit(res);
        }
      if(set_ip)
        {
          data.ip_addr = ip;
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_ip_addr, &data);
          error_exit(res);
        }
      if(set_port)
        {
          data.ip_port = htons(port);
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_ip_port, &data);
          error_exit(res);
        }
      if(algo)
        {
          strncpy(data.string, algo, RSBAC_LIST_REPL_CRYPTALGO_LEN);
          data.string[RSBAC_LIST_REPL_CRYPTALGO_LEN - 1] = 0;
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_crypt_algo, &data);
          error_exit(res);
        }
      if(set_key)
        {
          memcpy(data.string, key, RSBAC_LIST_REPL_CRYPTKEY_LEN);
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_crypt_key, &data);
          error_exit(res);
          data.crypt_key_len = crypt_key_len;
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_crypt_key_len, &data);
          error_exit(res);
        }
      if(add_flags || remove_flags)
        {
          res = rsbac_list_repl_get_partner_item(ta_number, id, REPL_flags, &data);
          error_exit(res);
          data.flags &= ~remove_flags;
          data.flags |= add_flags;
          res = rsbac_list_repl_mod_partner(ta_number, id, REPL_flags, &data);
          error_exit(res);
        }
      exit(0);
    }
  else
    {
      use();
      return 1;
    }
  return (res);
}
